/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    scoreDetections.cpp
** AUTHOR(S):   Stephen Gould <sgould@stanford.edu>
**              Olga Russakovsky <olga@cs.stanford.edu>
** DESCRIPTION:
**   Given two XML object files, computes the precision and recall.
*****************************************************************************/

#include <cstdlib>
#include <string>
#include <limits>
#include <iomanip>
#include <fstream>
#include <map>
#include <vector>

#include "svlBase.h"
#include "svlVision.h"

using namespace std;

// type definitions and function prototypes ----------------------------------



// main ----------------------------------------------------------------------

void usage()
{
    cerr << SVL_USAGE_HEADER << endl;
    cerr << "USAGE: ./scoreDetections [OPTIONS] (<groundtruth xml> <results xml>)+\n";
    cerr << "OPTIONS:\n"
	 << "  -threshold <num>  :: threshold for positive classification (default 0.5)\n"
	 << "  -pr <filestem>    :: generate precision-recall curves\n"
	 << "  -o <filestem>     :: output detections (true and false)\n"
	 << "  -object <name>    :: specifies one object of interest\n"
         << SVL_STANDARD_OPTIONS_USAGE
	 << endl;
}

int main(int argc, char *argv[])
{
    // read commandline parameters
    const int NUM_REQUIRED_PARAMETERS = 2;

    const char *gtFilename = NULL;
    const char *xmlFilename = NULL;

    double threshold = 0.5;
    const char *prFilestem = NULL;
    const char *outputFilestem = NULL;
    const char *object = NULL;

    SVL_BEGIN_CMDLINE_PROCESSING(argc, argv)
      SVL_CMDLINE_REAL_OPTION("-threshold", threshold)
      SVL_CMDLINE_STR_OPTION("-pr", prFilestem)
      SVL_CMDLINE_STR_OPTION("-o", outputFilestem)
      SVL_CMDLINE_STR_OPTION("-object", object)

      // [[TODO: deprecate from here]]
      else if (!strcmp(*SVL_CMDLINE_ARGV, "-area")) {
	svlObjectDetectionAnalyzer::DEFAULT_AREA_RATIO = atof(*(++SVL_CMDLINE_ARGV));
	SVL_CMDLINE_ARGC--;
	SVL_OPTION_DEPRECATED("-area", "svlVision.svlObjectDetectionAnalyzer areaRatio")
      } else if (!strcmp(*SVL_CMDLINE_ARGV, "-noNonMax")) {
	svlObjectDetectionAnalyzer::USE_NONMAX = false;
	SVL_OPTION_DEPRECATED("-noNonMax", "svlVision.svlObjectDetectionAnalyzer useNonmax")
      } else if (!strcmp(*SVL_CMDLINE_ARGV, "-match")) {
	SVL_OPTION_DEPRECATED("-match", "svlVision.svlObjectDetectionAnalyzer includeAllFrames")
	SVL_LOG(SVL_LOG_WARNING, "(includeAllFrames is the OPPOSITE of -match, and it's "
		<< "now turned OFF by default, i.e. only matching frames are now included by default)");
      }
      // [[TODO: deprecate to here]]
 
    SVL_END_CMDLINE_PROCESSING(usage())
    
    if (SVL_CMDLINE_ARGC < NUM_REQUIRED_PARAMETERS) {
	usage();
	return -1;
    }

    // Accumulate counts per object class, e.g. objectScores["mug"]
    // holds the score of every mug detection and whether it is a
    // true positive (1) or not (0).

    svlObjectDetectionAnalyzer *d;
    if (object == NULL) {
      d = new svlMultObjectsDetectionAnalyzer();
    } else {
      d = new svlObjectDetectionAnalyzer(string(object));
    }

    for (unsigned i = 0; i < (unsigned)(SVL_CMDLINE_ARGC / 2); i++) {
	gtFilename = SVL_CMDLINE_ARGV[2 * i];
	xmlFilename = SVL_CMDLINE_ARGV[2 * i + 1];

	SVL_LOG(SVL_LOG_VERBOSE, "Reading from " << gtFilename << " and " << xmlFilename);
	
	if (!d->loadGt(gtFilename)) continue;
	if (!d->loadDetections(xmlFilename)) continue;

 	// output detections
	if (outputFilestem) {
	  string filename = string(outputFilestem) + string(".") + toString(i) + string(".xml");
	  d->outputDetections(filename.c_str(), threshold);
	}

        // accumulate counts
        d->accumulateScores();
    }
   
    // show results
    d->printResults((float)threshold);

    // construct PR curve
    if (prFilestem != NULL) {
	SVL_LOG(SVL_LOG_VERBOSE, "Generating precision-recall curves...");
	d->createPRcurves();
	d->writePRcurves(prFilestem);
	SVL_LOG(SVL_LOG_VERBOSE, "...done");
    }

    delete d;

    return 0;
}


