/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    svlClassifierResponseList.h
** AUTHOR(S):   Olga Russakovsky <olga@cs.stanford.edu>
**                based on applications by Stephen Gould <sgould@stanford.edu>
** DESCRIPTION:
**  Provides helper functions for analyzing the output of any 2-class
**  classifier
**
*****************************************************************************/

#pragma once

#include <cassert>
#include <iostream>
#include <vector>
#include <map>

#include "cv.h"
#include "cxcore.h"
#include "ml.h"

#include "svlBase.h"
#include "svlPRcurve.h"

using namespace std;

// [[TODO: clean this up]]

static const float DEFAULT_THRESHOLD_DELTA = 0.01f;

// svlClassifierResponseList

// Holds the pair <feature value, 1[correct classification]> However,
// positive detection might be weighted differently than negative
// ones, that's why the second field is a float, not an int
typedef vector<pair<float, float> > svlClassifierResponseList;

// num (from 0 to values.size()) is contained in cache[i][j] iff
// i = 1[values[num].first > best_MI_threshold] (so this feature fires on the example), and
// j = 1[values[num].second > 0] (so this example belongs to the positive class)
typedef vector<int>** svlClassifierResponseCache;

// if the weight of positive samples is known ahead of time
// (and is not 1), can use the second version
void insertScore(svlClassifierResponseList &responses, float score, bool correct);
void insertScore(svlClassifierResponseList &responses, float score, float weight);
  
// this will change the weight of the positive detection such that
// overall positive and negative examples will have the same weight
void normalizeScores(svlClassifierResponseList &responses);
  
// given the current accumulated scores, creates a PR curve
void createPRcurve(const svlClassifierResponseList &responses, svlPRcurve &curve);

// same as above, except a rough approximation
void createDiscretizedPRcurve(const svlClassifierResponseList &responses,
			      svlPRcurve &curve, 
			      float dthreshold = DEFAULT_THRESHOLD_DELTA);

// returns a string summarizing the responses
string summary(const svlClassifierResponseList &responses, float threshold);

// gets the mutual information between this feature and the class
// label (at a particular threshold)
float computeMI(const svlClassifierResponseList &responses, float threshold);

// gets the maximum mutual information between this feature and the
// class label, over all possible thresholds; returns the pair of
// <maxMIattained, threshold>
pair<float, float> computeMaxMI(const svlClassifierResponseList &responses);

// gets the threshold at which the mutual information between this
// feature and the class label is the highest, and thresholds all
// scores to be 1[score > threshold], and class labels to be 1[label >
// 0]; returns the cache list, which can be used to speed up the
// computeJointMI function; if given the right pointers,
// can also return the best_MI and best_threshold
svlClassifierResponseCache
thresholdResponsesForMaxMI(const svlClassifierResponseList &responses,
			   float *best_MI = NULL, float *best_threshold = NULL);

void deleteClassifierResponseCache(svlClassifierResponseCache &cache);

// gets the I(X, Y; C), i.e. the joint mutual information of the two features
// assumes that both responses were already thresholded to achieve the maximum
// mutual information between the feature and the class
// further assumes that the scores were entered in the same order (so correspond
// to the same objects)
float computeJointMI(const svlClassifierResponseCache &cache_X,
		     const svlClassifierResponseCache &cache_Y);


