/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    svlPRCurve.cpp
** AUTHOR(S):   Olga Russakovsky <olga@cs.stanford.edu>
**              Ian Goodfellow <ia3n@cs.stanford.edu>
**
*****************************************************************************/

#include <cassert>
#include <iostream>
#include <fstream>
#include <vector>
#include <sys/types.h>
#include <limits>

#include "svlBase.h"
#include "svlVision.h"

#define VERBOSE
#undef DEBUG
//#define DEBUG

using namespace std;


// --------------------------------------------------------

bool svlPRpoint::operator<(const svlPRpoint &a) const {
  switch(_sort_flag) {
    // sorts smallest to largest
  case BY_RECALL:
    if (_recall == a._recall)
      return _pr < a._pr;
    return _recall < a._recall;
    
  case BY_PRECISION:
    if (_pr == a._pr)
      return _recall < a._recall;
    return _pr < a._pr;
    
    // sorts largest to smallest
  case BY_FSCORE:
    return _Fscore > a._Fscore;
    
  default:
    SVL_ASSERT_MSG(false, "Can't compare PR points if no sort flag is set");
  }
  return false; // to suppress warnings
}

void svlPRpoint::print(ofstream &ofs) const {
  ofs << _threshold << ", " << _recall << ", " << _pr << "\n";
}

float svlPRpoint::getFscore() const {
  if (_pr == 0 || _recall == 0 || contains_nan())
    return 0;
  else
    return 2.0f / (1.0f/_recall + 1.0f/_pr);
}

void svlPRpoint::saveFscore() {
  _Fscore = getFscore();
}

float svlPRpoint::getPRcontribution(const svlPRpoint *prev) const {
  switch(_sort_flag) {
  case BY_RECALL:
    if (prev)	return ((_pr + prev->_pr) / 2) * (_recall - prev->_recall);
    return _pr * _recall; // extend the point out to the edge of the graph
    
  case BY_PRECISION:
    if (prev) return ((_recall + prev->_recall) / 2) * (_pr - prev->_pr);
    return _pr * _recall; // extend the point out to the edge of the graph
    
  default:
    SVL_ASSERT_MSG(false, "The points on the PR curve should be sorted" <<
		   " by either precision or recall, not " << (int)_sort_flag);
  }
  return 0; // to suppress warnings
}

bool svlPRpoint::isDuplicate(const svlPRpoint &next) const {
  switch(_sort_flag) {
  case BY_RECALL:
    return (_recall == next._recall);
  case BY_PRECISION:
    return (_pr == next._pr);
  default:
    SVL_ASSERT_MSG(false, "The points on the PR curve should be sorted" <<
		   " by either precision or recall, not " << (int)_sort_flag);
  }
  return false; // to suppress warnings
}



// --------------------------------------------------------

void writePRcurve(const char *filename, const svlPRcurve &curve) {
  ofstream ofs(filename);
  assert(!ofs.fail());
  ofs << "THRESHOLD, RECALL, PRECISION\n";
  for (unsigned i = 0; i < curve.size(); i++)
    curve[i].print(ofs);
  ofs.close();
}

bool readPRcurve(const char *filename, svlPRcurve &curve) {
  curve.clear();

  ifstream ifs(filename);
  if (ifs.fail()) {
    SVL_LOG(SVL_LOG_ERROR, "...failed to read " << filename);
    return false;
  }
  char buffer[512];
  ifs.getline(buffer, 512);
  while (ifs.good()) {
    ifs.getline(buffer, 512);
    float t, r, p;
    sscanf(buffer, "%f, %f, %f", &t, &r, &p);
    svlPRpoint pr(t, r, p);
    if (!pr.contains_nan())
      curve.push_back(pr);
  }

  if (curve.size() == 0)
    SVL_LOG(SVL_LOG_WARNING, "No non-NAN points on the curve");
  
  ifs.close();

  return true;
}

float getHighestFscore(const svlPRcurve &curve) {
  float result = 0;
  for (unsigned i = 0; i < curve.size(); i++) {
    float Fscore = curve[i].getFscore();
    if (Fscore > result) result = Fscore;
  }
  return result;
}


float getPRarea(const svlPRcurve &curve, svlPRpointSort_t s) {

  if (s == BY_FSCORE) {
    SVL_LOG(SVL_LOG_FATAL, "Invalid sorting option BY_FSCORE in getPRarea");
    return 0;
  }

  if (s == NONE)
    return (getPRarea(curve, BY_RECALL) + getPRarea(curve, BY_PRECISION)) / 2;
  
  svlPRcurve points = curve; // will need to alter it

  // should filter out all the nan points before now
  if (points.size() == 0) return 0;

  if (points.size() == 1) {
    points[0]._sort_flag = (svlPRpointSort_t)s;
    return points[0].getPRcontribution();
  }

  // sort by 's'
  for (unsigned i = 0; i < points.size(); i++)
    points[i]._sort_flag = s;
  sort(points.begin(), points.end());

  // delete the duplicates at each precision/recall level, depending
  // on what we're sorting by
  for (int i = (int)points.size() - 2; i >= 0; --i) {
    if (points[i].isDuplicate(points[i+1]))
      points.erase(points.begin() + i);
  }

  // extend the prCurve out to the edge of the graph first, then
  // incrementally add trapezoids
  float area = points[0].getPRcontribution();
  for (unsigned i = 1; i < points.size(); i++)
    area += points[i].getPRcontribution(&(points[i-1]));

  return area;
}

//Finds all possible split points in posScores and negScores. populates thresholds, tp, fp, tn, and fn
void tryThresholds(vector<double> & posScores, vector<double> & negScores, vector<double> & thresholds, vector<unsigned int> & tp, vector<unsigned int> & fp, vector<unsigned int> & tn, vector<unsigned int> & fn)
{
  int numPosExamples = posScores.size();
  int numNegExamples = negScores.size();
  

  //sort both score vectors;
  sort (posScores.begin(), posScores.end());
  sort (negScores.begin(), negScores.end());




//try each threshold from the two vectors and export resulting (TDR,FDR) pair;
  int i = 0, j =0;

  bool iIsValid = i < numPosExamples;
  bool jIsValid = j < numNegExamples;

  assert(tp.size() == fp.size() == tn.size() == fn.size());

  while (iIsValid || jIsValid)
    {
      tp.push_back(numPosExamples - i);
      fp.push_back(numNegExamples - j);
      tn.push_back(j);
      fn.push_back(i);
      thresholds.push_back( max(posScores[i], negScores[j]) );
      
      if (iIsValid && jIsValid)
	{
	  bool iCanAdvance = i + 1 < numPosExamples;
	  bool jCanAdvance = j + 1 < numNegExamples;

	  if (iCanAdvance && jCanAdvance)
	    {
	      if (posScores[i+1] == negScores[j+1])
		{
		  i++;
		  j++;
		}
	      else if (posScores[i+1] < negScores[j+1])
		{
		  i++;
		}
	      else // negScores[i+1] < posScores[j+1]
		{
		  j++;
		}
	    }
	  else
	    {
	      //If both are equal, it's not possible to bump only one off
	      if (posScores[i] == negScores[i])
		{
		  i++;
		  j++;
		}
	      else
		{ //Otherwise, bump just one off by pushing its iterator past the end
		  if (iCanAdvance)
		    j++;
		  else
		    i++;
		}
	    }
	}
      else if (iIsValid)
	{
	  i++;
	}
      else if (jIsValid)
	{
	  j++;
	}

      iIsValid = i < numPosExamples;
      jIsValid = j < numNegExamples;
		   
    }
}


svlPRcurve makePRcurve(vector<double> & posScores, vector<double> & negScores)
{
  vector<unsigned int> tp, fp, tn, fn;
  vector<double> thresholds;
  svlPRcurve rval;

  tryThresholds(posScores,negScores,thresholds,tp,fp,tn,fn);

  for (unsigned i = 0; i < tp.size(); i++)
      rval.push_back(svlPRpoint(thresholds[i], double(tp[i])/double(tp[i]+fn[i]), double(tp[i])/double(tp[i]+fp[i])));

  return rval;
}
















